#!/usr/bin/env node

const   available = require('available-versions'),
        commander = require('commander'),
        detectInstalled = require('detect-installed'),
        { prompt } = require('inquirer');

const   { init } = require('./logic');

let     availableReactNativeVersions = [];

/* ----------------------------------------------------------------------------------------------- */

commander
    .version('0.1.7')
    .description('React Native Project Creator');

commander
    .command('init')
    .description('Init React Native Project')
    .action(() => {
        detectInstalled('@react-native-community/cli').then((exists) => {
            if(exists) {
                console.log('')

                getReactNativeVersions()
            } else {
                console.log("\x1b[31m", '\nYou need install @react-native-community/cli globally first before using react-native-project-creator init.\n', "\x1b[0m");
            }
        });
    });

commander.parse(process.argv);

const initQuestions = [
    {
        type : 'input',
        name : 'name',
        message : 'Enter project name :',
        validate : function validateName(name){
            const fs = require('fs-extra');

            if(name.trim() === '') {
                console.log("\x1b[31m", '\nProject name cannot be empty!\n', "\x1b[0m");
            } else if (fs.existsSync('./' + name)) {
                console.log("\x1b[31m", '\nDirectory already exists!\n', "\x1b[0m");
            } else if(/\s/g.test(name)) {
                console.log("\x1b[31m", '\nCannot using whitespace in project name!\n', "\x1b[0m");
            } else {
                return name.trim() !== '';
            }
        }
    },
    {
        type : 'list',
        name : 'version',
        message : 'Enter react native version to use :',
        choices : availableReactNativeVersions,
        pageSize : 10
    },
    {
        type : 'input',
        name : 'package_name',
        message : 'Enter package name :',
        default : 'Default Package Name',
        validate : function validatePackageName(package_name) {
            if(package_name == 'Default Package Name') {
                return true;
            } else if(package_name.trim() === '') {
                console.log("\x1b[31m", '\nPackage name cannot be empty!\n', "\x1b[0m");

                return false;
            } else if(/\s/g.test(package_name)) {
                console.log("\x1b[31m", '\nCannot using whitespace in package name!\n', "\x1b[0m");

                return false;
            } else {
                return true;
            }
        }
    }
];

function getReactNativeVersions() {
    let query = {
        name: 'react-native'
    };
    
    available(query).then(function (result) {
        for(let i = result.versions.length - 1; i >= 0; i--) {
            availableReactNativeVersions.push(result.versions[i]);
        }

        if(availableReactNativeVersions.length > 0) {
            prompt(initQuestions).then(inputs => {
                console.log('');
    
                init(inputs);
            });
        }
    });
}