const promise = require('bluebird');
const exec = require('child_process').exec;
const fs = require('fs-extra');
const replace = require('replace-in-file');

/* ----------------------------------------------------------------------------------------------- */

/**
 * @function  [initReactNativeProject]
 * @returns {Void}
 */

function init(parameters) {
    const initProcess = exec(
        'react-native init ' + parameters.name + ' --version="' + parameters.version + '"'
    );

    promiseFromChildProcess(initProcess).then(function (result) {
        if (fs.existsSync('./' + parameters.name + '/index.js')) {
            createStartFile(parameters);
        }
    }, function (err) {
        
    });

    initProcess.stdout.on('data', function(data) {
        console.log(data);
    });

    initProcess.stderr.on('data', function(data) {
        console.log("\x1b[31m", data, "\x1b[0m");
    });
};

function createStartFile(parameters) {
    const createStartFileProcess = exec(
        'cd ' + parameters.name +
        ' && rm -r index.js' +
        ' && rm -r App.js' +
        ' && git init'
    );

    promiseFromChildProcess(createStartFileProcess).then(function (result) {
        copyFiles(parameters);

        if(parameters.package_name != 'Default Package Name') {
            replaceAndroid(parameters);

            replaceIOS(parameters, () => replaceTvOS(parameters));

            replaceTvOSPlist(parameters);
        }
    }, function (err) {
        
    });
}

function promiseFromChildProcess(child) {
    return new promise(function (resolve, reject) {
        child.addListener("error", reject);
        child.addListener("exit", resolve);
    });
}

function copyFiles(parameters) {
    fs.copy(__dirname + "/src/index.js", './' + parameters.name + '/index.js', (err) => {
        if (err) throw err;
    });

    fs.copy(__dirname + '/src/colors.js', './' + parameters.name + '/colors.js', (err) => {
        if (err) throw err;
    });
}

function replaceAndroid(parameters) {
    const options = {
        files: [
            './' + parameters.name + '/android/app/src/main/java/com/' + parameters.name + '/MainActivity.java',
            './' + parameters.name + '/android/app/src/main/java/com/' + parameters.name + '/MainApplication.java',
            './' + parameters.name + '/android/app/src/main/AndroidManifest.xml',
            './' + parameters.name + '/android/app/BUCK',
            './' + parameters.name + '/android/app/build.gradle'
        ],
        from: new RegExp("com." + parameters.name.toLowerCase(), "g"),
        to: parameters.package_name,
    };

    replace(options)
    .then(changedFiles => {
        fs.rename(
            './' + parameters.name + '/android/app/src/main/java/com/' + parameters.name + '/MainActivity.java',
            './' + parameters.name + '/android/app/src/main/java/MainActivity.java',
            (err) => {
                if (err) throw err;

                fs.rename(
                    './' + parameters.name + '/android/app/src/main/java/com/' + parameters.name + '/MainApplication.java',
                    './' + parameters.name + '/android/app/src/main/java/MainApplication.java',
                    (err) => {
                        if (err) throw err;

                        let path = parameters.package_name.replace(/\./g, '/'),
                            mkdirCommand = "";

                        for(let i = 0; i < path.split('/').length; i++) {
                            mkdirCommand += " && mkdir " + path.split('/')[i] + " && cd " + path.split('/')[i];
                        }
                        
                        let adjustPackageName = exec(
                            'cd ' + parameters.name + '/android/app/src/main/java/' +
                            ' && rm -r com' +
                            mkdirCommand
                        );

                        promiseFromChildProcess(adjustPackageName).then(function (result) {
                            fs.rename(
                                './' + parameters.name + '/android/app/src/main/java/MainActivity.java',
                                './' + parameters.name + '/android/app/src/main/java/' + path + '/MainActivity.java',
                                (err) => {
                                    if (err) throw err;

                                    fs.rename(
                                        './' + parameters.name + '/android/app/src/main/java/MainApplication.java',
                                        './' + parameters.name + '/android/app/src/main/java/' + path + '/MainApplication.java',
                                        (err) => {
                                            if (err) throw err;

                                        }
                                    );
                                }
                            );
                        }, function (err) {
                            
                        });
                    }
                );
            }
        );
    })
    .catch(error => {
        console.error('Error occurred:', error);
    });
}

function replaceIOS(parameters, additionalAction) {
    const options = {
        files: [
            './' + parameters.name + '/ios/' + parameters.name + '.xcodeproj/project.pbxproj'
        ],
        from: /org\.reactjs\.native\.example\.\$\(PRODUCT_NAME:rfc1034identifier\)/g,
        to: parameters.package_name,
    }

    replace(options)
    .then(changedFiles => {
        additionalAction();
    })
    .catch(error => {
        console.error('Error occurred:', error);
    });
}

function replaceTvOS(parameters) {
    const options = {
        files: [
            './' + parameters.name + '/ios/' + parameters.name + '.xcodeproj/project.pbxproj'
        ],
        from: new RegExp("com.facebook.REACT." + parameters.name, "g"),
        to: parameters.package_name,
    }

    replace(options)
    .then(changedFiles => {

    })
    .catch(error => {
        console.error('Error occurred:', error);
    });
}

function replaceTvOSPlist(parameters) {
    const options = {
        files: [
            './' + parameters.name + '/ios/' + parameters.name + '-tvOS/Info.plist',
            './' + parameters.name + '/ios/' + parameters.name + '-tvOSTests/Info.plist',
        ],
        from: /org\.reactjs\.native\.example\.\$\(PRODUCT_NAME:rfc1034identifier\)/g,
        to: parameters.package_name + '-tvOS',
    }

    replace(options)
    .then(changedFiles => {

    })
    .catch(error => {
        console.error('Error occurred:', error);
    });
}

module.exports = {
    init
};